CQ.Ext.ns('SMARTLING.ui.submission');

SMARTLING.ui.submission.SubmissionAdmin = CQ.Ext.extend(CQ.Ext.Viewport, {
    constructor: function (config) {
        // ------------------------------------------------------------
        //      Refreshing statuses
        // ------------------------------------------------------------
        var ajaxQueue = [];
        var ajaxQueueCounter = 0;

        function queueProgressRequest(divId, submission) {
            var progressValues = [divId, submission.fileUri, submission.sourcePath, submission.configType, submission.locale];
            ajaxQueue.push(progressValues);
            if (ajaxQueue.length < 6) {
                loadSubmissionProgress(progressValues);
            }
        }

        function setProgress(divId, progress) {
            var progressBarCellId = divId + "-progress";
            CQ.Ext.fly(progressBarCellId).update("");
            new CQ.Ext.ProgressBar({
                renderTo: progressBarCellId,
                value: progress
            }).show();
        }

        function setWordCount(divId, wordCount) {
            var wordCountCellId = divId + "-wordCount";
            // toString is because update(0) clears the cell
            CQ.Ext.fly(wordCountCellId).update(wordCount.toString());
        }

        function loadSubmissionProgress(progressValues) {
            CQ.Ext.Ajax.request({
                method: 'GET',
                url: '/services/smartling/submission/progressStatus.json',
                params: {
                    fileUri: progressValues[1],
                    sourcePath: progressValues[2],
                    configType: progressValues[3],
                    locale: progressValues[4]
                },
                success: function (response) {
                    var submissionStatus = JSON.parse(response.responseText);

                    setProgress(progressValues[0], submissionStatus.progress);
                    setWordCount(progressValues[0], submissionStatus.wordCount);
                    ajaxQueueCounter = ajaxQueueCounter + 1;
                    if (ajaxQueueCounter < ajaxQueue.length) {
                        loadSubmissionProgress(ajaxQueue[ajaxQueueCounter]);
                    } else {
                        ajaxQueue = [];
                        ajaxQueueCounter = 0;
                    }
                }
            });
        }

        function stopProgressRequests() {
            ajaxQueueCounter = 0;
            ajaxQueue = [];
        }

        // ------------------------------------------------------------
        //      Setting up UI
        // ------------------------------------------------------------
        config.grid.columns.push({
            header: "Progress",
            dataIndex: 'progress',
            sortable: false,
            width: 100,
            renderer: function (value, metaData, record) {
                (function () {
                    setProgress(record.id, (value));
                }).defer(25);

                var id = record.id + "-progress";
                return '<div id="' + id + '" style="line-height: 20px">' + value + '</div>';
            }
        });
        config.grid.columns.push({
            header: "Word Count",
            dataIndex: 'wordCount',
            sortable: false,
            width: 90,
            align: 'right',
            renderer: function (value, metaData, record) {
               queueProgressRequest(record.id, record.data);
                var id = record.id + "-wordCount";
                return '<div id="' + id + '">' + value + '</div>';
            }
        });

        var submissionGrid = new SMARTLING.ui.submission.SubmissionGrid(config.grid);

        submissionGrid.addBeforeReloadListener(stopProgressRequests);

        SMARTLING.ui.submission.SubmissionAdmin.superclass.constructor.call(this, {
            layout: "border",
            items: [
                {
                    xtype: 'panel',
                    layout: 'border',
                    region: 'center',
                    defaults: {
                        split: true
                    },
                    items: [
                        new SMARTLING.ui.common.PageHeader()
                        , {
                            region: 'west',
                            collapsible: true,
                            hideCollapseTool: true,
                            collapseMode: 'mini',
                            width: 300,
                            minSize: 300,
                            maxSize: 300,
                            items: [
                                new SMARTLING.ui.submission.SubmissionFilterPanel(submissionGrid)
                            ]
                        }, {
                            collapsible: false,
                            region: 'center',
                            padding: 0,
                            layout: "fit",
                            items: [{
                                xtype: 'panel',
                                layout: 'fit',
                                region: 'center',
                                margins: '0 0 5 5',
                                border: false,
                                items: submissionGrid
                            }]
                        }]
                }
            ]
        });
    }
});
CQ.Ext.reg('smartling-submissions', SMARTLING.ui.submission.SubmissionAdmin);
